/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.cassandra.utils.progress.jmx;

import java.util.concurrent.ExecutorService;

import static org.apache.cassandra.concurrent.ExecutorFactory.Global.executorFactory;

/**
 * Holds dedicated executor for JMX event handling. Events will internally queued by ArrayNotificationBuffer,
 * synchronized by an exclusive write lock, which makes a shared single thread executor desirable.
 */
public final class JMXBroadcastExecutor
{

    private JMXBroadcastExecutor() { }

    public final static ExecutorService executor = executorFactory().sequential("JMX");

}
